--[[

Addon that reminds the player to recast self-buffs when they are not present, or
getting ready to expire. You have to cast the buff yourself in the usual way
but at least you won't get caught out having forgotten to cast anything on
yourself. It also works with the tracker for herbs and minerals. Uses ACE2. 
I've seen other mods that do the same job to various degrees, but they all seem
rather heavyweight. I wanted something small and light, and this is it. Buffs are
checked every 30 seconds.

To use: /br

/br /aksbr and /aksbuffremind are all slash commands for this addon

combatwombat/aszune/eu created May 2007                        

]]

-- $Log: AKSBuffRemind.lua,v $
-- Revision 1.11  2007-07-31 06:19:22  aksen
-- Removed code that is no longer needed
--
-- Revision 1.10  2007-07-30 22:22:39  aksen
-- Better tracker detection
--
-- Revision 1.9  2007-07-10 22:58:36  aksen
-- remove spellarray {} as no longer used
--
-- Revision 1.8  2007-07-10 22:53:53  aksen
-- Check value is true before putting message about tracker up
--
-- Revision 1.7  2007-06-10 19:33:52  aksen
-- check for UnitIsDead("pet")
--
-- Revision 1.6  2007-06-09 23:10:41  aksen
-- Removed unwanted AKSBuffRemindTooltip code
--
-- Revision 1.5  2007-06-09 23:06:21  aksen
-- Use SpecialEvents-Aura-2.0 for buff detection
--
-- Revision 1.4  2007-06-09 21:10:51  aksen
-- Updated comment text
--
-- Revision 1.3  2007-06-09 20:46:08  aksen
-- Known Buffs only
--
-- Revision 1.2  2007-06-09 20:45:07  aksen
-- All spells from spellbook
--
-- Revision 1.1.1.1  2007-06-09 20:43:04  aksen
-- original version
--

AKSBuffRemind = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
local dewdrop = AceLibrary:GetInstance("Dewdrop-2.0")
local babble  = AceLibrary("Babble-Spell-2.2")
local L       = AceLibrary("AceLocale-2.2"):new("AKSBuffRemind")
local aura    = AceLibrary("SpecialEvents-Aura-2.0")

function AKSBuffRemind:OnInitialize()
  self.eventid = 0
  self.args = {
    type = "group",
    args = {
      notify = {
        type = 'range',
        name = L["Notify before buff expires"],
        desc = L["Notify me this number of seconds before the buff expires"],
        usage = L["notify N"],
        min = 0,
        max = 300,
        isPercent = false,
        step = 1,
        get = function() return self.db.char.notify end,
        set = function(n) self.db.char.notify = n end,
        error = L["Value outside the range 0..300"],
      },
      interval = {
        type = 'range',
        name = L["Check Interval"],
        desc = L["How often to check in seconds"],
        usage = L["interval N"],
        min = 10,
        max = 300,
        isPercent = false,
        step = 1,
        get = function() return self.db.char.interval end,
        set = function(n) 
          self:CancelScheduledEvent(self.eventid) 
          self.db.char.interval = n 
          self.eventid = self:ScheduleRepeatingEvent(self.Timer, n, self)
        end,
        error = L["Value outside the range 10..300"],
      },
      showinchat = {
        type = 'toggle',
        name = L["Show in Chat"],
        desc = L["If set, your message will be displayed in the General chat window."],
        get = function() return self.db.char.showinchat end,
        set = function(s) self.db.char.showinchat = s if s then self.frame:Hide() self.db.char.locked = true end end,
      },
      font = {
        type = 'text',
        name = L["Font size"],
        desc = L["Set the font size for the message frame"],
        validate = {L["small"], L["normal"], L["large"], L["huge"]},
        get = function() return self.db.char.font end,
        set = function(v)
          self.db.char.font = v
          self:SetFont()
        end,
        disabled = function() return (self.db.char.showinchat == true) end,
        -- disabled = function() return not (self.db.char.display == "Default") end,
      },
      lock = {
        type = 'toggle',
        name = L["Lock"],
        desc = L["Lock the frame"],
        get = function() return self.db.char.locked end,
        set = function(v)
          self.db.char.locked = v
         if v then
           self.frame:Hide()
         else
           self.frame:Show()
         end
       end,
       disabled = function() return (self.db.char.showinchat == true) end,
       -- disabled = function() return not (self.db.char.display == "Default") end,
     },
     test = {
       type = 'execute',
       name = L["Test"],
       desc = L["Run test script"],
       func = 'Test',
     },
     color = {
       type = 'color',
       name = L["Message color"],
       desc = L["Set the color of the message"],
       get = function() return self.db.char.msgColor.r, self.db.char.msgColor.g, self.db.char.msgColor.b end,
       set = function(r, g, b) self.db.char.msgColor = { r = r, g = g, b = b } end,
     },
     config = {
       type = 'execute',
       name = L["GUI Config"],
       desc = L["Graphical options menu"],
       guiHidden = true,
       func = function()
         dewdrop:Open(UIParent, 'children', function() dewdrop:FeedAceOptionsTable(self.args) end, 'cursorX', false, 'cursorY', false)
       end,
      },
      -- header = {
      --   name = L["--- Spell list follows ---"],
      --   type = 'header',
      --   order = 110 
      -- },
      close = {     
        order = 500,
        type = 'execute',
        desc = L["Close this menu"],
        name = L["Close"],
        func = function() dewdrop:Close() end
      },
      bufflist = {     
        type = 'group',
        hasArrow = true,
        desc = L["The list of buffs to watch"],
        name = L["Buff List"],
        args = {
          -- this is filled in when the spell book is read
        },
      }
    }
  }
  self:RegisterDB("AKSBuffRemindDB", "AKSBuffRemindDBPC")
  self:RegisterChatCommand(L["Slash-Commands"], self.args, "AKSBUFFREMIND")
  SlashCmdList["AKSBUFFREMIND"] = function() dewdrop:Open(UIParent, 'children', function() dewdrop:FeedAceOptionsTable(self.args) end, 'cursorX', false, 'cursorY', false) end
  self:RegisterDefaults("char",
  {
    notify = 60,
    interval = 30,
    font = "large",
    anchor = false,
    locked = true,
    showinchat = false,
    chatColor = { r = 1, g = 1, b = 0 },
    msg = true,
    msgColor = { r = 1, g = 0, b = 0 },
    -- display = "Default",
    t = false,
    spelllist = {},
  })
  --local t = {
  --  type = 'toggle',
  --  name = 'testname1',
  --  desc = 'test desc 1',
  --  get  = function() return self.db.char.t end,
  --  set  = function(v) self.db.char.t = v end,
  --}
  --self.args.args['temp'] = t
end

-- function AKSBuffRemind:StartUp()
-- end

function AKSBuffRemind:OnEnable()
  -- if AceLibrary("AceEvent-2.0"):IsFullyInitialized() then
  --   self:StartUp()
  -- else
  --   self:RegisterEvent("AceEvent_FullyInitialized", "StartUp")
  -- end
  self.eventid = self:ScheduleRepeatingEvent(self.Timer, self.db.char.interval, self)
  self:RegisterEvent("PLAYER_LOGIN")    
  self:RegisterEvent("PLAYER_LOGOUT")    
  self:RegisterEvent("SPELLS_CHANGED")
  self:CreateFrames()
  self:SetFont()
  -- here's a list of things known to be buffs. this is used as a filter when
  -- reading the spell book otherwise a really huge listing gets generated
  self.bufflist = {}
  self.trackerlist = {}
  -------------------------------------------------------------------------
  --
  -- I'm not 100% sure about this list, please let me know of anything
  -- that is missing
  --
  -------------------------------------------------------------------------
  -- druid
  self.bufflist[babble["Mark of the Wild"]] = true
  self.bufflist[babble["Gift of the Wild"]] = true
  self.bufflist[babble["Thorns"]] = true
  -- mage
  self.bufflist[babble["Arcane Intellect"]] = true
  self.bufflist[babble["Arcane Brilliance"]] = true
  self.bufflist[babble["Ice Armor"]] = true
  self.bufflist[babble["Frost Armor"]] = true
  self.bufflist[babble["Mage Armor"]] = true
  self.bufflist[babble["Molten Armor"]] = true
  -- priest
  self.bufflist[babble["Power Word: Fortitude"]] = true
  self.bufflist[babble["Improved Power Word: Fortitude"]] = true
  self.bufflist[babble["Prayer of Fortitude"]] = true
  -- warlock
  self.bufflist[babble["Fel Armor"]] = true
  self.bufflist[babble["Demon Armor"]] = true
  self.bufflist[babble["Demon Skin"]] = true
  self.bufflist[babble["Unending Breath"]] = true
  self.bufflist[babble["Detect Greater Invisibility"]] = true
  self.bufflist[babble["Detect Invisibility"]] = true
  self.bufflist[babble["Detect Lesser Invisibility"]] = true
  self.bufflist[babble["Soul Link"]] = true
  self.bufflist[babble["Shadow Ward"]] = true
  self.bufflist[babble["Soulstone Resurrection"]] = true
  -- hunter
  self.bufflist[babble["Trueshot Aura"]] = true
  self.bufflist[babble["Rapid Fire"]] = true
  self.bufflist[babble["Aspect of the Hawk"]] = true
  self.bufflist[babble["Aspect of the Monkey"]] = true
  self.bufflist[babble["Aspect of the Wild"]] = true
  self.bufflist[babble["Aspect of the Beast"]] = true
  self.bufflist[babble["Aspect of the Cheetah"]] = true
  self.bufflist[babble["Aspect of the Pack"]] = true
  self.bufflist[babble["Aspect of the Viper"]] = true
  -- shaman
  self.bufflist[babble["Lightning Shield"]] = true
  self.bufflist[babble["Water Shield"]] = true
  self.bufflist[babble["Earth Shield"]] = true
  self.bufflist[babble["Rockbiter Weapon"]] = true
  self.bufflist[babble["Frostbrand Weapon"]] = true
  self.bufflist[babble["Flametongue Weapon"]] = true
  self.bufflist[babble["Windfury Weapon"]] = true
  self.bufflist[babble["Water Breathing"]] = true
  -- warrior
  self.bufflist[babble["Battle Shout"]] = true
  self.bufflist[babble["Commanding Shout"]] = true
  self.bufflist[babble["Berserker Rage"]] = true
  self.bufflist[babble["Bloodrage"]] = true
  -- paladin
  self.bufflist[babble["Righteous Fury"]] = true
  self.bufflist[babble["Holy Shield"]] = true
  self.bufflist[babble["Blessing of Might"]] = true
  self.bufflist[babble["Greater Blessing of Might"]] = true
  self.bufflist[babble["Blessing of Wisdom"]] = true
  self.bufflist[babble["Greater Blessing of Wisdom"]] = true
  self.bufflist[babble["Blessing of Salvation"]] = true
  self.bufflist[babble["Greater Blessing of Salvation"]] = true
  self.bufflist[babble["Blessing of Kings"]] = true
  self.bufflist[babble["Greater Blessing of Kings"]] = true
  self.bufflist[babble["Blessing of Sanctuary"]] = true
  self.bufflist[babble["Greater Blessing of Sanctuary"]] = true
  self.bufflist[babble["Blessing of Light"]] = true
  self.bufflist[babble["Greater Blessing of Light"]] = true
  self.bufflist[babble["Blessing of Freedom"]] = true
  self.bufflist[babble["Blessing of Protection"]] = true
  self.bufflist[babble["Seal of Command"]] = true 
  self.bufflist[babble["Seal of Justice"]] = true
  self.bufflist[babble["Seal of Light"]] = true
  self.bufflist[babble["Seal of Righteousness"]] = true
  self.bufflist[babble["Seal of Wisdom"]] = true
  self.bufflist[babble["Seal of the Crusader"]] = true
  self.bufflist[babble["Devotion Aura"]] = true
  self.bufflist[babble["Retribution Aura"]] = true
  self.bufflist[babble["Concentration Aura"]] = true
  self.bufflist[babble["Shadow Resistance Aura"]] = true
  self.bufflist[babble["Frost Resistance Aura"]] = true
  self.bufflist[babble["Fire Resistance Aura"]] = true
  self.bufflist[babble["Sanctity Aura"]] = true
  self.bufflist[babble["Crusader Aura"]] = true
  -- trackables
  self.trackerlist[babble["Find Minerals"]] = true
  self.trackerlist[babble["Find Herbs"]] = true
  self.trackerlist[babble["Find Treasure"]] = true
  self.trackerlist[babble["Track Humanoids"]] = true
  self.trackerlist[babble["Track Beasts"]] = true
  self.trackerlist[babble["Track Undead"]] = true
  self.trackerlist[babble["Track Hidden"]] = true
  self.trackerlist[babble["Track Elementals"]] = true
  self.trackerlist[babble["Track Demons"]] = true
  self.trackerlist[babble["Track Giants"]] = true
  self.trackerlist[babble["Track Dragonkin"]] = true
  self.trackerlist[babble["Sense Demons"]] = true
  self.trackerlist[babble["Sense Undead"]] = true
  self:Print(L["Monitoring buffs, /br config for options"])
  self.textures = {}
  self.textures[babble["Find Herbs"]]       = "Interface\\Icons\\INV_Misc_Flower_02"
  self.textures[babble["Find Minerals"]]    = "Interface\\Icons\\Spell_Nature_Earthquake"
  self.textures[babble["Find Treasure"]]    = "Interface\\Icons\\Racial_Dwarf_FindTreasure"
  self.textures[babble["Track Beasts"]]     = "Interface\\Icons\\Ability_Tracking"
  if UnitClass("player") == L["Druid"] then
    self.textures[babble["Track Humanoids"]]  = "Interface\\Icons\\Ability_Tracking"
  else
    self.textures[babble["Track Humanoids"]]  = "Interface\\Icons\\Spell_Holy_PrayerOfHealing"
  end
  self.textures[babble["Track Hidden"]]     = "Interface\\Icons\\Ability_Stealth"
  self.textures[babble["Track Elementals"]] = "Interface\\Icons\\Spell_Frost_SummonWaterElemental"
  self.textures[babble["Track Undead"]]     = "Interface\\Icons\\Spell_Shadow_DarkSummoning"
  self.textures[babble["Track Demons"]]     = "Interface\\Icons\\Spell_Shadow_SummonFelHunter"
  self.textures[babble["Track Giants"]]     = "Interface\\Icons\\Ability_Racial_Avatar"
  self.textures[babble["Track Dragonkin"]]  = "Interface\\Icons\\INV_Misc_Head_Dragon_01"
  self.textures[babble["Sense Undead"]]     = "Interface\\Icons\\Spell_Holy_SenseUndead"
  self.textures[babble["Sense Demons"]]     = "Interface\\Icons\\Spell_Shadow_Metamorphosis" 
end

function AKSBuffRemind:OnDisable()
  self:CancelScheduledEvent(self.eventid) 
end

function AKSBuffRemind:ReadSpellBook()
  local n 
  local i = 1
  n = GetSpellName(i, BOOKTYPE_SPELL)
  while n do 
    if self.bufflist[n] or self.trackerlist[n] then
      local t = {
        order = 200,
        passValue = n,
        type = 'toggle',
        name = n,
        desc = n,
        get  = function(n) return self.db.char.spelllist[n] end,
        set  = function(n,v) self.db.char.spelllist[n] = v end,
      }
      -- uncomment the next line to have the buffs shown on the initial menu
      -- also uncomment the header in the main options table if you do this
      -- self.args.args[n] = t
      self.args.args.bufflist.args[n] = t
    end
    i = i + 1
    n = GetSpellName(i, BOOKTYPE_SPELL)
  end
end

function AKSBuffRemind:PLAYER_LOGIN()
  -- self:Print("I have logged in")
  self:ReadSpellBook()
end

function AKSBuffRemind:PLAYER_LOGOUT()
  -- self:Print("I have logged out")
end

function AKSBuffRemind:Timer()
  local buffindex, i, key, value, timeleft, buffname, found, dead
  dead = UnitIsGhost("player") or UnitIsDead("player")
  for key, value in pairs(self.db.char.spelllist) do
    if value then
      found = false
      buffindex = aura:UnitHasBuff("player", key)
      if buffindex and buffindex > 0 then
        found = true
        timeleft = GetPlayerBuffTimeLeft(buffindex)
        local t = timeleft
        local sc = self:SpecialCase(key)
        local d = DEFAULT_CHAT_FRAME
        if t > 0 and t <= self.db.char.notify and not IsMounted() and not sc and not dead then
          local color = self.db.char.msgColor
          if self.db.char.showinchat then
            self:CustomPrint(color.r, color.g, color.b, d, nil, " ", key .. L[" expires soon"])
          else
            AKSBuffRemindMessageFrame:AddMessage(key..L[" expires soon"],color.r,color.g,color.b,1,0.5)
            -- UIErrorsFrame:AddMessage(key .. L[" expires soon"], 1, 1, 0, 5)
          end
          PlaySound("TellMessage")
        end
      end
      if not found and not IsMounted() and not self:SpecialCase(key) and not dead then
        local color = self.db.char.msgColor
        local d = DEFAULT_CHAT_FRAME
        if self.db.char.showinchat then
          self:CustomPrint(color.r, color.g, color.b, d, nil, " ", L["Recast "] .. key)
        else
          AKSBuffRemindMessageFrame:AddMessage(L["Recast "] .. key, color.r, color.g, color.b, 1, 0.5)
          -- UIErrorsFrame:AddMessage(L["Recast "] .. key, 1, 0, 0, 5.0)
        end
        PlaySound("Deathbind Sound")
      end
    end
    -- warn about the tracker if we are mounted because it can be cast while mounted
    if value and not dead and self:IsTrackableSkill(key) and not self:IsTrackerSpecialCase(key) then
      local texture = GetTrackingTexture()
      -- self:Print("key is " .. key .. " textures[key] is " .. self.textures[key] .. " texture is " .. texture)
      if not texture or not self.textures[key] or self.textures[key] ~= texture then
        local color = self.db.char.msgColor
        local d = DEFAULT_CHAT_FRAME
        local msg = L["Recast "] .. key
        if self.db.char.showinchat then
          self:CustomPrint(color.r, color.g, color.b, d, nil, " ", msg)
        else
          AKSBuffRemindMessageFrame:AddMessage(msg, color.r, color.g, color.b, 1, 0.5)
        end
        PlaySound("Deathbind Sound")
      end
    end
  end
end

function AKSBuffRemind:SPELLS_CHANGED()
  self:ReadSpellBook()
end
 
-- returns false if a spell needs a pet, eg soul link, but no pet is out
-- not sure if there are any other cases that need checking
function AKSBuffRemind:SpecialCase(s)
  -- no soul link warnings if no pet
  if s == babble["Soul Link"] and (not UnitExists("pet") or UnitIsDead("pet")) then
    return true
  end
  -- trackable skills use the minimap tracker and do not appear as buffs
  if self:IsTrackableSkill(s) then
    return true
  end
  return false
end

function AKSBuffRemind:IsTrackerSpecialCase(s)
  -- for a druid don't warn about track humanoids if not in cat form
  if string.lower(UnitClass("player")) == "druid"  and s == babble["Track Humanoids"] then
    local icon, name, active
    -- 1 = Bear/Dire Bear Form
    -- 2 = Aquatic Form
    -- 3 = Cat Form
    -- 4 = Travel Form
    -- 5 = Moonkin Form 
    icon, name, active = GetShapeshiftFormInfo(3)
    if not active then
      return true
    end
  end    
  return false
end

function AKSBuffRemind:IsTrackableSkill(s)
  local k, v
  for k, v in pairs(self.trackerlist) do
    if v and k == s then
      return true
    end
  end
  return false
end

function AKSBuffRemind:CreateFrames()
  self.frame = CreateFrame("Frame", "AKSBuffRemindAnchorFrame", UIParent)
  self.frame:EnableMouse(true)
  self.frame:SetMovable(true)
  self.frame:SetClampedToScreen(true)
  self.frame:SetWidth(600)
  self.frame:SetHeight(100)
  self.frame:SetPoint("BOTTOM", UIParent, "CENTER", 0, 0)
  self.frame:SetFrameStrata("LOW")

  self.frame.title = self.frame:CreateFontString(nil, "ARTWORK", "TextStatusBarText")
  self.frame.title:SetText("AKSBuffRemind")
  self.frame.title:SetPoint("TOP", 0, 0)
        
  self.frame:SetBackdrop({ 
    bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
    edgeFile = nil, tile = true, tileSize = 16, edgeSize = 16, 
   insets = { left = 5, right = 5, top = 5, bottom = 5 }
  })
  self.frame:SetBackdropColor(0, 0, 0, 0.6)

  self.frame:SetScript("OnMouseDown", function()
  if (arg1 == "LeftButton") then
    this:StartMoving()
    elseif (arg1 == "RightButton") then
      dewdrop:Register(self.frame, 'children', function() dewdrop:FeedAceOptionsTable(self.args) end, 'point', "TOPRIGHT", 'relativePoint', "BOTTOMLEFT", 'cursorX', true, 'cursorY', true)
    end
  end)
  self.frame:SetScript("OnMouseUp", function()
    if (arg1 == "LeftButton") then
      this:StopMovingOrSizing()
      self:SavePosition()
    end
  end)

  if (self.db.char.locked) then self.frame:Hide() end

  self:LoadPosition()

  self.msgframe = CreateFrame("MessageFrame", "AKSBuffRemindMessageFrame")
  self.msgframe:SetMovable(true)
  self.msgframe:SetWidth(1024)
  self.msgframe:SetHeight(180)
  self.msgframe:SetFadeDuration(1.5)
  self.msgframe:SetTimeVisible(5)
  self.msgframe:ClearAllPoints()
  self.msgframe:SetPoint("TOP", "AKSBuffRemindAnchorFrame", "TOP", 0, -20)
  self.msgframe:SetInsertMode("TOP")
  self.msgframe:SetFrameStrata("HIGH")
  self.msgframe:CreateFontString(nil, nil, "NumberFontNormalHuge")
end

function AKSBuffRemind:SavePosition()
  local x, y = self.frame:GetLeft(), self.frame:GetTop() - UIParent:GetTop()

  if (not self.db.char.pos) then 
    self.db.char.pos = {}
  end

  self.db.char.pos.x = x
  self.db.char.pos.y = y
end

function AKSBuffRemind:LoadPosition()
  if (self.db.char.pos) then
    local x = self.db.char.pos.x
    local y = self.db.char.pos.y

    self.frame:ClearAllPoints()
    self.frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x, y)
  else
    self.frame:ClearAllPoints()
    self.frame:SetPoint("BOTTOM", UIParent, "CENTER")
  end
end

function AKSBuffRemind:SetFont()
  if (self.db.char.font == L["small"]) then
    AKSBuffRemindMessageFrame:SetFontObject(NumberFontNormalSmall)
  elseif (self.db.char.font == L["large"]) then
    AKSBuffRemindMessageFrame:SetFontObject(NumberFontNormalLarge)
  elseif (self.db.char.font == L["huge"]) then
    AKSBuffRemindMessageFrame:SetFontObject(NumberFontNormalHuge)
  else
    AKSBuffRemindMessageFrame:SetFontObject(NumberFontNormal)
  end
end

function AKSBuffRemind:Test()
  local color = self.db.char.msgColor
  if self.db.char.showinchat then
    self:CustomPrint(color.r, color.g, color.b, DEFAULT_CHAT_FRAME, nil, " ", L["Sample Text"])
  else
    AKSBuffRemindMessageFrame:AddMessage(L["Sample Text"], color.r, color.g, color.b, 1, 0.5)
  end
end

-- lose leading, trailing and multiple blanks
function AKSBuffRemind:Trim(s)
  s = string.gsub(s, "^%s+","")
  s = string.gsub(s, "%s+$","")
  s = string.gsub(s, "%s%s+"," ")
  return s
end

